/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <pkgmgr-info.h>
#include <app_contents.h>
#include <app_define.h>
#include <app_debug.h>

#include "define.h"
#include "data/app.h"

struct app_data {
	char *appid;
	char *pkgid;
	char *name;
	char *icon;
	char *pkgtype;
	bool removable;
	struct color_data icon_bg;
	struct color_data text_bg;
};

static inline void _set_default_color(struct color_data *c)
{
	c->r = 255;
	c->g = 255;
	c->b = 255;
	c->a = 255;
}

static int _get_app_data_foreach(pkgmgrinfo_appinfo_h handle, void *data)
{
	Eina_List **list;
	struct app_data *adata;
	char *appid, *pkgid, *name, *icon, *pkgtype;
	pkgmgrinfo_pkginfo_h pkginfo_h;
	bool nodisplay;
	bool removable = true;

	if (!data)
		return -1;

	list = (Eina_List **)data;

	if (pkgmgrinfo_appinfo_is_nodisplay(handle, &nodisplay) != PMINFO_R_OK)
		return -1;

	if (nodisplay)
		return 0;

	if (pkgmgrinfo_appinfo_get_appid(handle, &appid) != PMINFO_R_OK)
		return -1;

	if (pkgmgrinfo_appinfo_get_pkgid(handle, &pkgid) != PMINFO_R_OK)
		return -1;

	if (pkgmgrinfo_appinfo_get_label(handle, &name) != PMINFO_R_OK)
		return 0;

	if (pkgmgrinfo_appinfo_get_icon(handle, &icon) != PMINFO_R_OK)
		return 0;

	if (pkgmgrinfo_pkginfo_get_usr_pkginfo(pkgid, getuid(),
				&pkginfo_h) != PMINFO_R_OK)
		return 0;

	if (pkgmgrinfo_pkginfo_is_removable(pkginfo_h, &removable) != PMINFO_R_OK)
		return 0;

	if (pkgmgrinfo_pkginfo_get_type(pkginfo_h, &pkgtype) != PMINFO_R_OK) {
		pkgmgrinfo_pkginfo_destroy_pkginfo(pkginfo_h);
		return 0;
	}

	adata = calloc(1, sizeof(*adata));
	if (!adata) {
		_ERR("Calloc failed.");
		pkgmgrinfo_pkginfo_destroy_pkginfo(pkginfo_h);
		return -1;
	}

	if (appid) {
		if (!strcmp(appid, APP_ID_MEDIAHUB))
			goto OUT;

		adata->appid = strdup(appid);

		if (!strcmp(appid, APP_ID_BROWSER)) {
			_set_default_color(&adata->icon_bg);
			_set_default_color(&adata->text_bg);

			adata->name = STR_BROWSER;
		} else {
			app_contents_get_color(name, &adata->icon_bg, &adata->text_bg);

			if (name)
				adata->name = strdup(name);
		}
	}
	if (pkgid)
		adata->pkgid = strdup(pkgid);
	if (icon)
		adata->icon = strdup(icon);
	if (pkgtype)
		adata->pkgtype = strdup(pkgtype);

	adata->removable = removable;

	*list = eina_list_append(*list, adata);

OUT:
	pkgmgrinfo_pkginfo_destroy_pkginfo(pkginfo_h);

	return 0;
}

Eina_List *get_app_list(void)
{
	Eina_List *list = NULL;

	pkgmgrinfo_appinfo_get_usr_installed_list(_get_app_data_foreach,
			getuid(), &list);

	return list;
}

char *get_app_id(struct app_data *adata)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return NULL;
	}

	return adata->appid;
}

char *get_pkg_id(struct app_data *adata)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return NULL;
	}

	return adata->pkgid;
}

char *get_app_name(struct app_data *adata)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return NULL;
	}

	return adata->name;
}

char *get_app_icon(struct app_data *adata)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return NULL;
	}

	return adata->icon;
}

char *get_pkg_type(struct app_data *adata)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return NULL;
	}

	return adata->pkgtype;
}

bool get_app_removable(struct app_data *adata)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return NULL;
	}

	return adata->removable;
}

bool get_app_favorite(struct app_data *adata,  bool *isfavorite)
{
	int r;

	if (!adata || !adata->appid) {
		_ERR("Invalid argument.");
		return false;
	}

	r = app_contents_favorite_check(CONTENTS_APP, adata->appid,
			isfavorite);
	if (r != APP_CONTENTS_ERROR_NONE)
		return false;

	return true;
}

bool set_app_favorite(struct app_data *adata, bool isfavorite)
{
	int r;

	if (!adata || !adata->appid) {
		_ERR("Invalid argument.");
		return false;
	}

	r = app_contents_favorite_set(CONTENTS_APP, adata->appid, isfavorite);
	if (r != APP_CONTENTS_ERROR_NONE)
		return false;

	return true;
}

bool get_app_icon_bg_color(struct app_data *adata,
		int *r, int *g, int *b, int *a)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return false;
	}

	*r = adata->icon_bg.r;
	*g = adata->icon_bg.g;
	*b = adata->icon_bg.b;
	*a = adata->icon_bg.a;

	return true;
}

bool get_app_text_bg_color(struct app_data *adata,
		int *r, int *g, int *b, int *a)
{
	if (!adata) {
		_ERR("Invalid argument.");
		return false;
	}

	*r = adata->text_bg.r;
	*g = adata->text_bg.g;
	*b = adata->text_bg.b;
	*a = adata->text_bg.a;

	return true;
}

void free_app_list(Eina_List *list)
{
	struct app_data *adata;

	if (!list)
		return;

	EINA_LIST_FREE(list, adata) {
		if (!adata)
			continue;

		free(adata->appid);
		free(adata->name);
		free(adata->icon);
		free(adata);

		adata = NULL;
	}
}

Eina_List *get_app_favorite_list(Eina_List *list)
{
	Eina_List *l, *fav_list = NULL;
	struct app_data *adata;
	bool isfavorite;

	if (!list)
		return NULL;

	EINA_LIST_FOREACH(list, l, adata) {
		if (adata && get_app_favorite(adata, &isfavorite) && isfavorite)
			fav_list = eina_list_append(fav_list, adata);
	}

	return fav_list;
}

static int _sort_by_a_to_z(const void *first_data, const void *second_data)
{
	struct app_data *adata1, *adata2;

	adata1 = (struct app_data *)first_data;
	adata2 = (struct app_data *)second_data;

	if (!adata1 || !get_app_name(adata1))
		return 1;

	if (!adata2 || !get_app_name(adata2))
		return -1;

	return strcmp(get_app_name(adata1), get_app_name(adata2));
}

static int _sort_by_z_to_a(const void *first_data, const void *second_data)
{
	struct app_data *adata1, *adata2;

	adata1 = (struct app_data *)first_data;
	adata2 = (struct app_data *)second_data;

	if (!adata1 || !get_app_name(adata1))
		return -1;

	if (!adata2 || !get_app_name(adata2))
		return 1;

	return strcmp(get_app_name(adata2), get_app_name(adata1));
}

static void _recent_list_foreach(gpointer data, gpointer user_data)
{
	struct recent_data *rdata;
	struct app_data *adata;
	Eina_List **list;
	Eina_List *l = NULL;

	if (!data || !user_data) {
		_ERR("Data is NULL.");
		return;
	}

	list = user_data;
	rdata = data;

	if (!rdata->id)
		return;

	EINA_LIST_FOREACH(*list, l, adata) {
		if (adata && get_app_id(adata) &&
				!strcmp(get_app_id(adata), rdata->id)) {
			*list = eina_list_promote_list(*list, l);
			break;
		}
	}
}

static void _sort_by_recently_used(Eina_List **list)
{
	GList *recent_l = NULL;
	struct app_data *adata;
	Eina_List *l = NULL;
	int r;

	r = app_contents_get_recent_list(CONTENTS_APP, -1, &recent_l);
	if (r != APP_CONTENTS_ERROR_NONE) {
		_ERR("Get recent list failed.");
		return;
	}

	if (!recent_l)
		return;

	recent_l = g_list_reverse(recent_l);
	g_list_foreach(recent_l, _recent_list_foreach, list);

	/* The 'Web Browser' should be at first when 'Recent' option selected */
	EINA_LIST_FOREACH(*list, l, adata) {
		if (adata && get_app_id(adata) &&
				!strcmp(get_app_id(adata), APP_ID_BROWSER)) {
			*list = eina_list_promote_list(*list, l);
			break;
		}
	}
}

bool get_sorted_app_list(Eina_List **list, enum sort_type type)
{
	if (!list) {
		_ERR("Invalid argument.");
		return false;
	}

	switch (type) {
	case SORT_RECENT:
		_sort_by_recently_used(list);
		break;

	case SORT_A_TO_Z:
		*list = eina_list_sort(*list, 0, _sort_by_a_to_z);
		break;

	case SORT_Z_TO_A:
		*list = eina_list_sort(*list, 0, _sort_by_z_to_a);
		break;

	default:
		_ERR("Unhandled sort type.");
		return false;
	}

	return true;
}
