/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_control.h>
#include <app_debug.h>
#include <package_manager.h>

#include "define.h"
#include "utils.h"

Evas_Object *utils_add_window(const char *name)
{
	Evas_Object *win;

	if (!name) {
		_ERR("Invalid argument.");
		return NULL;
	}

	win = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (!win) {
		_ERR("elm_win_add failed.");
		return NULL;
	}

	elm_win_title_set(win, name);
	elm_win_focus_highlight_enabled_set(win, EINA_TRUE);

	evas_object_show(win);

	return win;
}

Evas_Object *utils_add_layout(Evas_Object *parent, const char *group,
		Eina_Bool win_resize)
{
	Evas_Object *layout;

	if (!parent || !group) {
		_ERR("Invalid argument.");
		return NULL;
	}

	layout = elm_layout_add(parent);
	if (!layout) {
		_ERR("elm_layout_add failed.");
		return NULL;
	}

	elm_layout_file_set(layout, EDJEFILE, group);

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);

	if (win_resize)
		elm_win_resize_object_add(parent, layout);

	return layout;
}

Evas_Object *utils_add_box(Evas_Object *parent, const char *part,
		Eina_Bool horizontal, Evas_Coord padding_h,
		Evas_Coord padding_v)
{
	Evas_Object *box;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	box = elm_box_add(parent);
	if (!box) {
		_ERR("elm_box_add failed.");
		return NULL;
	}

	elm_box_horizontal_set(box, horizontal);
	elm_box_padding_set(box, ELM_SCALE_SIZE(padding_h),
			ELM_SCALE_SIZE(padding_v));

	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);

	if (part)
		elm_object_part_content_set(parent, part, box);
	else
		elm_object_content_set(parent, box);

	evas_object_show(box);

	return box;
}

Evas_Object *utils_add_button(Evas_Object *parent, const char *part,
		const char *text, const char *style)
{
	Evas_Object *btn;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	btn = elm_button_add(parent);
	if (!btn) {
		_ERR("elm_button_add failed.");
		return NULL;
	}

	if (part)
		elm_object_part_content_set(parent, part, btn);
	if (text)
		elm_object_text_set(btn, text);
	if (style)
		elm_object_style_set(btn, style);

	evas_object_show(btn);

	return btn;
}

Evas_Object *utils_add_gengrid(Evas_Object *parent, Eina_Bool horizontal,
		int width, int height)
{
	Evas_Object *grid;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	grid = elm_gengrid_add(parent);
	if (!grid) {
		_ERR("elm_gengrid_add failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(grid, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(grid, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_gengrid_multi_select_set(grid, EINA_FALSE);
	elm_gengrid_horizontal_set(grid, horizontal);
	elm_gengrid_align_set(grid, 0.0, 0.0);
	elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_gengrid_item_size_set(grid, ELM_SCALE_SIZE(width),
			ELM_SCALE_SIZE(height));
	elm_scroller_policy_set(grid, ELM_SCROLLER_POLICY_OFF,
			ELM_SCROLLER_POLICY_OFF);

	return grid;
}

Evas_Object *utils_add_table(Evas_Object *parent, const char *part,
		int padding_x, int padding_y)
{
	Evas_Object *table;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	table = elm_table_add(parent);
	if (!table) {
		_ERR("elm_table_add failed.");
		return NULL;
	}

	elm_table_padding_set(table, ELM_SCALE_SIZE(padding_x),
			ELM_SCALE_SIZE(padding_y));
	evas_object_show(table);

	if (part)
		elm_object_part_content_set(parent, part, table);

	return table;
}

static void _notify_timeout_cb(void *data, Evas_Object *obj, void *ei)
{
	if (!obj)
		return;

	evas_object_del(obj);
}

Evas_Object *utils_add_notify(Evas_Object *parent, const char *text,
		const char *noti_style, const char *label_style, int timeout)
{
	Evas_Object *notify, *lbl;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	notify = elm_notify_add(parent);
	if (!notify) {
		_ERR("elm_notify_add failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(notify, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_notify_align_set(notify, 0.0, 1.0);
	elm_notify_timeout_set(notify, timeout);

	if (noti_style)
		elm_object_style_set(notify, noti_style);

	lbl = elm_label_add(notify);
	if (!lbl) {
		_ERR("elm_label_add failed.");
		evas_object_del(notify);
		return NULL;
	}

	elm_object_content_set(notify, lbl);
	evas_object_show(lbl);

	if (label_style)
		elm_object_style_set(lbl, label_style);

	if (text)
		elm_object_text_set(lbl, text);

	evas_object_smart_callback_add(notify, SIG_TIMEOUT,
			_notify_timeout_cb, NULL);
	evas_object_show(notify);

	return notify;
}

Evas_Object *utils_add_popup(Evas_Object *parent, const char *style,
		const char *popup_title, const char *popup_text)
{
	Evas_Object *popup;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	popup = elm_popup_add(parent);
	if (!popup) {
		_ERR("elm_popup_add failed.");
		return NULL;
	}

	elm_popup_orient_set(popup, ELM_POPUP_ORIENT_CENTER);
	evas_object_show(popup);

	if (style)
		elm_object_style_set(popup, style);

	if (popup_title)
		elm_object_part_text_set(popup, PART_POPUP_TITLE, popup_title);

	if (popup_text)
		elm_object_text_set(popup, popup_text);

	return popup;
}

Evas_Object *utils_add_ctxpopup(Evas_Object *parent, int opt_size, int *opt_id,
		const char **opt_text, input_handler *opt_handler, void *data)
{
	Evas_Object *ctxpopup, *box, *btn = NULL, *firstbtn = NULL;
	int i;

	if (!parent || opt_size <= 0 || !opt_text) {
		_ERR("Invalid argument.");
		return NULL;
	}

	ctxpopup = elm_ctxpopup_add(parent);
	if (!ctxpopup) {
		_ERR("elm_ctxpopup_add failed.");
		return NULL;
	}

	box = utils_add_box(ctxpopup, NULL, EINA_FALSE, 0, 0);
	if (!box) {
		_ERR("Add box failed.");
		evas_object_del(ctxpopup);
		return NULL;
	}

	elm_object_style_set(ctxpopup, STYLE_CTXPOPUP);
	elm_object_content_set(ctxpopup, box);
	elm_ctxpopup_direction_priority_set(ctxpopup, 0, 0, 0, 0);
	evas_object_show(ctxpopup);

	for (i = 0; i < opt_size; i++) {
		btn = utils_add_button(box, NULL, opt_text[i],
				STYLE_OPTION_BTN);
		if (!btn) {
			_ERR("Add button failed.");
			evas_object_del(ctxpopup);
			return NULL;
		}

		elm_box_pack_end(box, btn);

		if (opt_handler)
			inputmgr_add_callback(btn, opt_id[i],
					opt_handler, data);

		if (i == 0)
			firstbtn = btn;
	}

	elm_object_focus_next_object_set(firstbtn, btn, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(btn, firstbtn, ELM_FOCUS_DOWN);

	return ctxpopup;
}

Evas_Object *utils_add_scroller(Evas_Object *parent, const char *part,
		Evas_Object *content)
{
	Evas_Object *scr;

	if (!parent) {
		_ERR("Invalid argument.");
		return NULL;
	}

	scr = elm_scroller_add(parent);
	if (!scr) {
		_ERR("elm_scroller_add failed.");
		return NULL;
	}

	elm_scroller_policy_set(scr,
			ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
	elm_object_focus_allow_set(scr, EINA_FALSE);

	if (part)
		elm_object_part_content_set(parent, part, scr);

	if (content)
		elm_object_content_set(scr, content);

	return scr;
}

bool utils_launch_app(const char *appid, const char *key, const char *value)
{
	app_control_h app_ctrl;
	int r;

	if (!appid) {
		_ERR("Invalid argument.");
		return false;
	}

	r = app_control_create(&app_ctrl);
	if (r != APP_CONTROL_ERROR_NONE) {
		_ERR("App control handle create failed.");
		return false;
	}

	r = app_control_set_operation(app_ctrl, APP_CONTROL_OPERATION_DEFAULT);
	if (r != APP_CONTROL_ERROR_NONE) {
		_ERR("App control set operation failed.");
		app_control_destroy(app_ctrl);
		return false;
	}

	if (key && value) {
		r = app_control_add_extra_data(app_ctrl, key, value);
		if (r != APP_CONTROL_ERROR_NONE) {
			_ERR("App control set extra data failed.");
			app_control_destroy(app_ctrl);
			return false;
		}
	}

	r = app_control_set_app_id(app_ctrl, appid);
	if (r != APP_CONTROL_ERROR_NONE) {
		_ERR("App control set app id failed.");
		app_control_destroy(app_ctrl);
		return false;
	}

	r = app_control_send_launch_request(app_ctrl, NULL, NULL);
	if (r != APP_CONTROL_ERROR_NONE) {
		_ERR("App control send launch request failed.");
		app_control_destroy(app_ctrl);
		return false;
	}

	app_control_destroy(app_ctrl);

	return true;
}

bool utils_uninstall_app(const char *pkgid, const char *pkgtype)
{
	package_manager_request_h request_h;
	int r, id;

	if (!pkgid || !pkgtype) {
		_ERR("Invalid argument.");
		return false;
	}

	r = package_manager_request_create(&request_h);
	if (r != PACKAGE_MANAGER_ERROR_NONE) {
		_ERR("Request handle create failed.");
		return false;
	}

	r = package_manager_request_set_type(request_h, pkgtype);
	if (r != PACKAGE_MANAGER_ERROR_NONE) {
		_ERR("Set pkg type failed.");
		package_manager_request_destroy(request_h);
		return false;
	}

	r = package_manager_request_set_mode(request_h,
			PACKAGE_MANAGER_REQUEST_MODE_QUIET);
	if (r != PACKAGE_MANAGER_ERROR_NONE) {
		_ERR("Set request mode failed.");
		package_manager_request_destroy(request_h);
		return false;
	}

	r = package_manager_request_uninstall(request_h, pkgid, &id);
	if (r != PACKAGE_MANAGER_ERROR_NONE) {
		_ERR("Uninstall failed.");
		package_manager_request_destroy(request_h);
		return false;
	}

	package_manager_request_destroy(request_h);

	return true;
}
