/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <viewmgr.h>
#include <gridmgr.h>
#include <inputmgr.h>
#include <app_contents.h>
#include <app_define.h>
#include <app_debug.h>

#include "define.h"
#include "utils.h"
#include "data/app.h"
#include "view.h"
#include "grid.h"

enum menu_type {
	MENU_MYAPPS = 0,
	MENU_APPSTORE,
};

const char *str_menu[] = {
	STR_MYAPPS,
	STR_APPSTORE,
	NULL
};

static int id_sort_opt[] = {
	SORT_RECENT,
	SORT_A_TO_Z,
	SORT_Z_TO_A,
};

const char *str_sort_opt[] = {
	STR_RECENT,
	STR_A_TO_Z,
	STR_Z_TO_A,
};

enum button_type {
	BTN_LIVETV = 0,
	BTN_FAVORITE,
	BTN_UNINSTALL,
	BTN_OK,
	BTN_CANCEL,
	BTN_MORE,
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *menu_btn[MENU_COUNT];
	Evas_Object *focused_menu;
	Evas_Object *sort_btn;
	Evas_Object *sort_popup;
	Evas_Object *uninstall_popup;
	Evas_Object *grid;
	Evas_Object *notify;
	Elm_Object_Item *focused_item;
	struct gridmgr *gmgr;
	Eina_List *app_list;
	enum sort_type sort_type;
};

/* Uninstallation - begin */
static void _mouse_move_cb(int id, void *data, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Move *ev);

static void _uninstall_selected(struct _priv *priv)
{
	struct app_data *adata = NULL;

	adata = elm_object_item_data_get(priv->focused_item);
	if (!adata) {
		_DBG("No app is selected");
		return;
	}

	if (!utils_uninstall_app(get_pkg_id(adata), get_pkg_type(adata))) {
		_ERR("Uninstall failed.");
		return;
	}

	viewmgr_update_view(VIEW_BASE, UPDATE_GRID, adata);
}

static void _popup_clicked_cb(int id, void *data, Evas_Object *obj)
{
	struct _priv *priv;

	if (!data)
		return;

	priv = data;

	evas_object_del(priv->uninstall_popup);

	if (id == BTN_OK)
		_uninstall_selected(priv);
}

static void _popup_key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!ev || !data) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (!strcmp(ev->keyname, KEY_BACK)
			|| !strcmp(ev->keyname, KEY_ESC))
		evas_object_del(priv->uninstall_popup);
}

static input_handler _popup_input_handler = {
	.mouse_move = _mouse_move_cb,
	.clicked = _popup_clicked_cb,
	.key_down = _popup_key_down_cb
};

static void _draw_uninstall_popup(struct _priv *priv)
{
	Evas_Object *popup = NULL;
	Evas_Object *btn1 = NULL;
	Evas_Object *btn2 = NULL;
	char str[SIZE_STR] = { 0, };
	char *name = NULL;
	struct app_data *adata = NULL;

	if (!priv) {
		_DBG("Invalid parameter");
		return;
	}

	adata = elm_object_item_data_get(priv->focused_item);
	if (!adata) {
		_DBG("No app is selected");
		return;
	}

	if (get_app_removable(adata) == false) {
		utils_add_notify(priv->base, "This application is not removable", STYLE_TOAST, STYLE_TOAST, 3);
		return;
	}

	name = get_app_name(adata);
	if (!name)
		return;

	snprintf(str, sizeof(str), "%s %s?", STR_UNINSTALL, name);

	popup = utils_add_popup(priv->base, NULL, STR_UNINSTALL, str);
	if (!popup) {
		_ERR("Add popup failed.");
		return;
	}

	btn1 = utils_add_button(popup, PART_BUTTON1, STR_CANCEL, NULL);
	if (!btn1) {
		_ERR("Add button failed.");
		evas_object_del(popup);
		return;
	}

	btn2 = utils_add_button(popup, PART_BUTTON2, STR_OK, NULL);
	if (!btn2) {
		_ERR("Add button failed.");
		evas_object_del(popup);
		return;
	}

	elm_object_focus_set(btn1, EINA_TRUE);

	priv->uninstall_popup = popup;

	inputmgr_add_callback(btn1, BTN_CANCEL, &_popup_input_handler, priv);
	inputmgr_add_callback(btn2, BTN_OK, &_popup_input_handler, priv);
}

/* Uninstallation - end */

static void _mouse_move_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!obj) {
		_ERR("Invalid argument.");
		return;
	}

	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static bool _draw_myapps_grid(struct _priv *priv)
{
	char str[SIZE_STR];
	int c;

	if (!priv->app_list) {
		elm_object_part_text_set(priv->base, PART_CONTENTS_TEXT,
				STR_NO_CONTENTS);
		elm_object_disabled_set(priv->sort_btn, EINA_TRUE);
		elm_object_part_content_unset(priv->base, PART_GENGRID);

		return true;
	}

	elm_object_part_text_set(priv->base, PART_CONTENTS_TEXT, "");

	if (!gridmgr_append_list(priv->gmgr, STR_MYAPPS, priv->app_list)) {
		_ERR("Gridmgr append list failed.");
		return false;
	}

	elm_object_part_content_set(priv->base, PART_GENGRID, priv->grid);
	priv->focused_item = elm_gengrid_first_item_get(priv->grid);

	c = eina_list_count(priv->app_list);
	snprintf(str, sizeof(str), " %d "STR_APPS, c);
	elm_object_part_text_set(priv->base, PART_APP_COUNT, str);

	return true;
}

static void _menu_focused_cb(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct _priv *priv;

	if (!data || !obj) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (priv->focused_menu == obj || !priv->grid)
		return;

	if (priv->focused_menu)
		elm_object_signal_emit(priv->focused_menu,
				SIG_UNSELECTED, SRC_MENU_BTN);

	switch (id) {
	case MENU_MYAPPS:
		elm_object_signal_emit(priv->base, SIG_VISIBLE, SRC_MYAPPS);
		_draw_myapps_grid(priv);
		break;

	case MENU_APPSTORE:
		elm_object_signal_emit(priv->base, SIG_INVISIBLE, SRC_MYAPPS);
		elm_object_part_text_set(priv->base, PART_CONTENTS_TEXT,
				STR_NOT_SUPPORTED);
		/* It will be implemented later. */
		break;

	default:
		_ERR("Invalid menu type.");
		return;
	}

	priv->focused_menu = obj;
}

static void _menu_key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data || !obj || !ev) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (id == MENU_MYAPPS && !strcmp(ev->keyname, KEY_DOWN)) {
		ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
		elm_object_focus_set(priv->grid, EINA_TRUE);
	} else if (!strcmp(ev->keyname, KEY_BACK)
			|| !strcmp(ev->keyname, KEY_ESC)) {
		ui_app_exit();
	}
}

static void _destroy_sort_ctxpopup(struct _priv *priv)
{
	if (!priv || !priv->sort_popup) {
		_ERR("Invalid argument.");
		return;
	}

	evas_object_hide(priv->sort_popup);
	evas_object_del(priv->sort_popup);
	priv->sort_popup = NULL;
}

static void _sort_option_selected(struct _priv *priv, int id, Evas_Object *obj)
{
	if (!priv->app_list || id <= SORT_INVALID || id >= SORT_MAX) {
		_ERR("Cannot sort");
		return;
	}

	if (!get_sorted_app_list(&priv->app_list, id)) {
		_ERR("Sort failed.");
		return;
	}

	priv->sort_type = id;

	_destroy_sort_ctxpopup(priv);
	elm_object_text_set(priv->sort_btn, elm_object_text_get(obj));
	gridmgr_append_list(priv->gmgr, STR_MYAPPS, priv->app_list);
}

static void _option_key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data || !obj || !ev) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (!strcmp(ev->keyname, KEY_BACK)
			|| !strcmp(ev->keyname, KEY_ESC)) {
		_destroy_sort_ctxpopup(priv);
	} else if (!strcmp(ev->keyname, KEY_ENTER)) {
		_sort_option_selected(priv, id, obj);
		priv->focused_item = elm_gengrid_first_item_get(priv->grid);
	}
}

static void _option_mouse_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	struct _priv *priv;

	if (!data || !obj || !ev) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	_sort_option_selected(priv, id, obj);
}

static input_handler _option_input_handler = {
	.mouse_move = _mouse_move_cb,
	.key_down = _option_key_down_cb,
	.mouse_down = _option_mouse_down_cb
};

static bool _draw_sort_option(struct _priv *priv)
{
	Evas_Object *ctxpopup;
	Evas_Coord x, y, w, h;

	elm_object_signal_emit(priv->sort_btn, SIG_SELECTED, SRC_SORT_BTN);

	ctxpopup = utils_add_ctxpopup(priv->base, SORT_COUNT, id_sort_opt,
			str_sort_opt, &_option_input_handler, priv);
	if (!ctxpopup) {
		_ERR("Add ctxpopup failed.");
		return false;
	}

	evas_object_geometry_get(priv->sort_btn, &x, &y, &w, &h);
	evas_object_move(ctxpopup, ELM_SCALE_SIZE(x + w / 2),
			ELM_SCALE_SIZE(y));

	priv->sort_popup = ctxpopup;

	return true;
}

static void _sort_key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data || !obj || !ev) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (!strcmp(ev->keyname, KEY_BACK)
			|| !strcmp(ev->keyname, KEY_ESC)
			|| !strcmp(ev->keyname, KEY_UP)) {
		ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;

		if (priv->focused_item)
			elm_object_item_focus_set(priv->focused_item,
					EINA_TRUE);
		else
			elm_object_focus_set(priv->focused_menu, EINA_TRUE);
	} else if (!strcmp(ev->keyname, KEY_ENTER)) {
		if (!_draw_sort_option(priv))
			_ERR("Draw sort ctxpopup failed.");

		elm_object_focus_set(priv->sort_popup, EINA_TRUE);
	}
}

static void _sort_mouse_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	struct _priv *priv;

	if (!data || !obj || !ev) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (!priv->app_list)
		return;

	if (!_draw_sort_option(priv))
		_ERR("Draw sort ctxpopup failed.");
}

static void _grid_mouse_move_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	Elm_Object_Item *it;

	if (!obj || !ev) {
		_ERR("Invalid argument.");
		return;
	}

	it = elm_gengrid_at_xy_item_get(obj, ev->cur.canvas.x,
			ev->cur.canvas.y, NULL, NULL);
	if (!it)
		return;

	if (!elm_object_item_focus_get(it))
		elm_object_item_focus_set(it, EINA_TRUE);
}

static void _grid_focused_cb(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	elm_object_signal_emit(priv->focused_menu, SIG_SELECTED, SRC_MENU_BTN);

	if (item)
		priv->focused_item = item;
}

static void _grid_realized_cb(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct app_data *adata;
	bool isfavorite;

	if (!data || !item) {
		_ERR("Invalid argument.");
		return;
	}

	adata = elm_object_item_data_get(item);

	if (get_app_favorite(adata, &isfavorite) && isfavorite)
		elm_object_item_signal_emit(item, SIG_FAVORITE,
				SRC_FAVORITE_ICON);
}

static void _grid_key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data || !ev) {
		_ERR("Invalid argument.");
		return;
	}
	priv = data;

	if (!strcmp(ev->keyname, KEY_BACK)
			|| !strcmp(ev->keyname, KEY_ESC)) {
		elm_object_focus_set(priv->focused_menu, EINA_TRUE);
	}
	else if (!strcmp(ev->keyname, KEY_MENU)) {
		_draw_uninstall_popup(priv);
	}
	else if (!strcmp(ev->keyname, KEY_ENTER)) {
		struct app_data *adata;
		adata = elm_object_item_data_get(priv->focused_item);

		if (!utils_launch_app(get_app_id(adata), NULL, NULL))
			_ERR("Launch app failed.");
	}
}

static input_handler _menu_input_handler = {
	.mouse_move = _mouse_move_cb,
	.focused = _menu_focused_cb,
	.key_down = _menu_key_down_cb
};

static input_handler _sort_input_handler = {
	.mouse_move = _mouse_move_cb,
	.key_down = _sort_key_down_cb,
	.mouse_down = _sort_mouse_down_cb
};

static input_handler _grid_input_handler = {
	.mouse_move = _grid_mouse_move_cb,
	.focused = _grid_focused_cb,
	.realized = _grid_realized_cb,
	.key_down = _grid_key_down_cb
};

static bool _draw_top_area(struct _priv *priv)
{
	Evas_Object *box, *btn;
	int i;

	elm_object_part_text_set(priv->base, PART_TOP_TITLE, STR_APPS);

	box = utils_add_box(priv->base, PART_MENU, EINA_TRUE,
			SIZE_MENU_BOX_PAD_H, SIZE_MENU_BOX_PAD_V);
	if (!box) {
		_ERR("Add box failed.");
		return false;
	}

	for (i = 0; i < MENU_COUNT; i++) {
		btn = utils_add_button(priv->base, NULL, str_menu[i],
				STYLE_MENU_BTN);
		if (!btn) {
			_ERR("elm_button_add failed.");
			evas_object_del(box);
			return false;
		}

		inputmgr_add_callback(btn, i, &_menu_input_handler, priv);

		elm_box_pack_end(box, btn);

		priv->menu_btn[i] = btn;
	}

	elm_object_focus_next_object_set(priv->menu_btn[0],
			priv->menu_btn[MENU_COUNT - 1], ELM_FOCUS_LEFT);
	elm_object_focus_next_object_set(priv->menu_btn[MENU_COUNT - 1],
			priv->menu_btn[0], ELM_FOCUS_RIGHT);
	elm_object_focus_next_object_set(priv->menu_btn[0], priv->menu_btn[0],
			ELM_FOCUS_DOWN);

	elm_object_focus_set(priv->menu_btn[0], EINA_TRUE);

	return true;
}

static bool _draw_sort_area(struct _priv *priv)
{
	Evas_Object *btn;

	btn = utils_add_button(priv->base, PART_SORT,
			str_sort_opt[priv->sort_type], STYLE_SORT_BTN);
	if (!btn) {
		_ERR("Add button failed.");
		return false;
	}

	inputmgr_add_callback(btn, 0, &_sort_input_handler, priv);

	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_DOWN);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_RIGHT);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_LEFT);
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(priv->grid, btn, ELM_FOCUS_DOWN);

	priv->sort_btn = btn;

	return true;
}

static Evas_Object *_add_grid(struct _priv *priv)
{
	struct grid_class *gclass;
	Evas_Object *grid;

	grid = utils_add_gengrid(priv->base, EINA_TRUE,
			SIZE_MYAPPS_GRID_WIDTH, SIZE_MYAPPS_GRID_HEIGHT);
	if (!grid) {
		_ERR("Add gengrid failed.");
		return NULL;
	}

	inputmgr_add_callback(grid, 0, &_grid_input_handler, priv);

	gclass = get_myapps_grid_class();
	if (!gclass) {
		_ERR("Get myapps grid class failed.");
		evas_object_del(grid);
		return NULL;
	}

	if (!gridmgr_add_grid(priv->gmgr, STR_MYAPPS, grid, gclass)) {
		_ERR("Gridmgr add grid failed.");
		evas_object_del(grid);
		return NULL;
	}

	return grid;
}

static void _draw_view_content(struct _priv *priv)
{
	if (!_draw_top_area(priv)) {
		_ERR("Draw top area failed.");
		return;
	}

	if (!_draw_myapps_grid(priv)) {
		_ERR("Draw myapps grid failed.");
		return;
	}

	if (!_draw_sort_area(priv))
		_ERR("Draw sort area failed.");
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	struct gridmgr *gmgr;
	Evas_Object *base, *grid;

	if (!win) {
		_ERR("Get window object failed.");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("Calloc failed.");
		return NULL;
	}

	base = utils_add_layout(win, GRP_VIEW_BASE, EINA_TRUE);
	if (!base) {
		_ERR("Add layout failed.");
		free(priv);
		return NULL;
	}

	gmgr = gridmgr_create();
	if (!gmgr) {
		_ERR("Create gridmgr failed.");
		goto end;
	}

	priv->win = win;
	priv->base = base;
	priv->gmgr = gmgr;
	priv->sort_type = SORT_RECENT;

	grid = _add_grid(priv);
	if (!grid) {
		_ERR("Add grid failed.");
		gridmgr_destroy(gmgr);
		goto end;
	}

	priv->grid = grid;
	priv->app_list = get_app_list();

	get_sorted_app_list(&priv->app_list, SORT_RECENT);

	if (!viewmgr_set_view_data(VIEW_BASE, priv)) {
		_ERR("Set view data failed.");
		gridmgr_destroy(gmgr);
		goto end;
	}

	_draw_view_content(priv);

	return base;

end:
	evas_object_del(base);
	free(priv);

	return NULL;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Get data failed.");
		return;
	}
	priv = data;

	if (priv->base)
		evas_object_show(priv->base);
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Get data failed.");
		return;
	}
	priv = data;

	if (priv->base)
		evas_object_hide(priv->base);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;
	struct app_data *adata;
	bool isfavorite;

	if (!view_data) {
		_ERR("Get data failed.");
		return;
	}
	priv = view_data;

	switch (update_type) {
	case UPDATE_FAVORITE:
		adata = data;
		if (!adata)
			return;

		if (!get_app_favorite(adata, &isfavorite))
			return;

		if (isfavorite)
			elm_object_item_signal_emit(priv->focused_item,
					SIG_FAVORITE, SRC_FAVORITE_ICON);
		else
			elm_object_item_signal_emit(priv->focused_item,
					SIG_UNFAVORITE, SRC_FAVORITE_ICON);

		break;

	case UPDATE_SHOW_TOAST:
		priv->notify = utils_add_notify(priv->base, (char *)data,
				STYLE_TOAST, STYLE_TOAST, SIZE_TOAST_TIMEOUT);

		break;

	case UPDATE_GRID:
		adata = data;
		if (!adata)
			return;

		priv->app_list = eina_list_remove(priv->app_list, adata);
		_draw_myapps_grid(priv);
		break;

	default:
		_ERR("Unhandled update type.");
	}
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Get data failed.");
		return;
	}
	priv = data;

	if (priv->gmgr)
		gridmgr_destroy(priv->gmgr);

	if (priv->base)
		evas_object_del(priv->base);

	free(priv);
}

static view_class _vclass = {
	.view_id = VIEW_BASE,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.update = _update,
	.destroy = _destroy,
};

view_class *view_base_get_vclass(void)
{
	return &_vclass;
}
